Package: wrMisc
Version: 2.1.0
Title: Analyze Experimental High-Throughput (Omics) Data
Authors@R: person("Wolfgang", "Raffelsberger", email = "w.raffelsberger@gmail.com", role = c("aut", "cre"))
Author: Wolfgang Raffelsberger [aut, cre]
Maintainer: Wolfgang Raffelsberger <w.raffelsberger@gmail.com>
Description: The efficient treatment and convenient analysis of experimental high-throughput (omics) data gets facilitated through this collection of diverse functions. 
  Several functions address advanced object-conversions, like manipulating lists of lists or lists of arrays, reorganizing lists to arrays or into separate vectors, merging of multiple entries, etc.  
  Another set of functions provides speed-optimized calculation of standard deviation (sd), coefficient of variance (CV) or standard error of the mean (SEM)  
  for data in matrixes or means per line with respect to additional grouping (eg n groups of replicates). 
  A group of functions facilitate dealing with non-redundant information, by indexing unique, adding counters to redundant or eliminating lines with respect redundancy in a given reference-column, etc. 
  Help is provided to identify very closely matching numeric values to generate (partial) distance matrixes for very big data in a memory efficient manner or to reduce the complexity of large data-sets by combining very close values. 
  Other functions help aligning a matrix or data.frame to a reference using partial matching or to mine an experimental setup to extract patterns of replicate samples.
  Many times large experimental datasets need some additional filtering, adequate functions are provided. 
  Convenient data normalization is supported in various different modes, parameter estimation via permutations or boot-strap as well as flexible testing of multiple pair-wise combinations using the framework of 'limma' is provided, too.
  Batch reading (or writing) of sets of files and combining data to arrays is supported, too. 
VignetteBuilder: knitr
Depends: R (>= 3.1.0)
Imports: grDevices, graphics, MASS, stats, utils
Suggests: BBmisc, boot, coin, data.table, data.tree, fdrtool,
        flexclust, knitr, limma, markdown, mixdist, NbClust,
        preprocessCore, qvalue, Rcpp, RColorBrewer, readxl, rmarkdown,
        som, stringi, VGAM, vsn, wrGraph
License: GPL-3
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-08 09:54:54 UTC; wraff
Repository: CRAN
Date/Publication: 2026-06-08 19:10:02 UTC
Built: R 4.5.2; ; 2026-06-08 21:56:07 UTC; unix
