# Generated by roxygen2: do not edit by hand

export(.addLetterWoLast)
export(.allRatioMatr1to2)
export(.allRatios)
export(.arrLstMean)
export(.arrLstSEM)
export(.asDF2)
export(.breakInSer)
export(.bringToCtr)
export(.checkArgNa)
export(.checkConsistentArrList)
export(.checkConvt2Vect)
export(.checkFactor)
export(.checkFileNameExtensions)
export(.checkLegendLoc)
export(.checkLmConfInt)
export(.checkRegrArguments)
export(.chooseGrpCol)
export(.combineListAnnot)
export(.compareByDiff)
export(.compareByLogRatio)
export(.compareByPPM)
export(.complCols)
export(.composeCallName)
export(.convertMatrToNum)
export(.convertNa)
export(.corDuplItemsByIncrem)
export(.cutAtSearch)
export(.cutStr)
export(.datSlope)
export(.doubleExt)
export(.extrNAneighb)
export(.extrNumHeadingCap)
export(.extrNumHeadingSepChar)
export(.filtSize)
export(.filterNetw)
export(.filterSw)
export(.findBorderOverlaps)
export(.firstMin)
export(.fuse2ArrBy2ndDim)
export(.getAmean)
export(.getAmean2)
export(.getMvalue2)
export(.getPWseparator)
export(.growTree)
export(.insp1dimByClustering)
export(.inspectHeader)
export(.keepCenter1d)
export(.keepFiniteCol)
export(.mayBeNum)
export(.medianSpecGrp)
export(.mergeMatrices)
export(.minDif)
export(.neigbDis)
export(.normConstSlope)
export(.normalize)
export(.pasteCols)
export(.plotCountPie)
export(.plusLowerCaps)
export(.predRes)
export(.raiseColLowest)
export(.removeCol)
export(.removeEmptyCol)
export(.replSpecChar)
export(.retain1stPart)
export(.rowGrpCV)
export(.rowGrpMeans)
export(.rowGrpSds)
export(.rowGrpSums)
export(.rowNorm)
export(.rowNormFact)
export(.scale01)
export(.scaleSpecGrp)
export(.scaleXY)
export(.seqCutStr)
export(.setLowestTo)
export(.sortMid)
export(.stackArray)
export(.summarizeCols)
export(.trimFromEnd)
export(.trimFromStart)
export(.trimLeft)
export(.trimRight)
export(.uniqueWName)
export(.vector2Matr)
export(XYToDiffPpm)
export(addBeforFileExtension)
export(addBeforeFileExtension)
export(adjBy2ptReg)
export(adjustUnitPrefix)
export(appendNR)
export(arrayCV)
export(asSepList)
export(blockNormalize)
export(buildTree)
export(cbindNR)
export(checkAvSd)
export(checkFilePath)
export(checkGrpOrder)
export(checkGrpOrderSEM)
export(checkSimValueInSer)
export(checkStrictOrder)
export(checkUnitPrefix)
export(checkVectLength)
export(chooseGroupNames)
export(cleanReplicates)
export(closeMatchMatrix)
export(coinPermTest)
export(colCVs)
export(colMedSds)
export(colSds)
export(colorAccording2)
export(combinatIntTable)
export(combineAsN)
export(combineByEitherFactor)
export(combineOverlapInfo)
export(combineRedBasedOnCol)
export(combineRedundLinesInList)
export(combineRedundLinesInListAcRef)
export(combineReplFromListToMatr)
export(combineSingleT)
export(completeArrLst)
export(concatMatch)
export(confInt)
export(contribToContigPerFrag)
export(conv01toColNa)
export(convColorToTransp)
export(convMatr2df)
export(convPairwiseSetup)
export(convToNum)
export(coordOfFilt)
export(correctToUnique)
export(correctWinPath)
export(countCloseToLimits)
export(countSameStartEnd)
export(cutArrayInCluLike)
export(cutAtMultSites)
export(cutToNgrp)
export(diffCombin)
export(diffPPM)
export(elimCloseCoord)
export(equLenNumber)
export(exclExtrValues)
export(exponNormalize)
export(extr1chan)
export(extrColsDeX)
export(extrNumericFromMatr)
export(extrSpcText)
export(extractLast2numericParts)
export(filt3dimArr)
export(filtSizeUniq)
export(filterLiColDeList)
export(filterList)
export(filterNetw)
export(findCloseMatch)
export(findHeadAndTail)
export(findRepeated)
export(findSimilFrom2sets)
export(findUsableGroupRange)
export(firstLineOfDat)
export(firstOfRepLines)
export(firstOfRepeated)
export(fuseAnnotMatr)
export(fuseCommonListElem)
export(fusePairs)
export(get1stOfRepeatedByCol)
export(getPWseparator)
export(getPairwiseSetup)
export(getValuesByUnique)
export(gitDataUrl)
export(htmlSpecCharConv)
export(indexGroupsFromPW)
export(keepCommonText)
export(levIndex)
export(linModelSelect)
export(linRegrParamAndPVal)
export(listBatchReplace)
export(listGroupsByNames)
export(lmSelClu)
export(lrbind)
export(makeMAList)
export(makeNRedMatr)
export(matchMatrixLinesToRef)
export(matchNamesWithReverseParts)
export(matchSampToPairw)
export(matr2list)
export(mergeMatrices)
export(mergeMatrixList)
export(mergeSelCol)
export(mergeSelCol3)
export(mergeVectors)
export(mergeW2)
export(minDiff)
export(moderTest2grp)
export(moderTestXgrp)
export(multiCharReplace)
export(multiMatch)
export(nFragments)
export(nFragments0)
export(nNonNumChar)
export(naOmit)
export(nonAmbiguousMat)
export(nonAmbiguousNum)
export(nonRedundLines)
export(nonredDataFrame)
export(normalizeThis)
export(numPairDeColNames)
export(orderMatrToRef)
export(organizeAsListOfRepl)
export(pVal2lfdr)
export(packageDownloadStat)
export(pairsAsPropensMatr)
export(partUnlist)
export(partialDist)
export(pasteC)
export(presenceFilt)
export(presenceGrpFilt)
export(protectSpecChar)
export(pwSeparatorList)
export(randIndFx)
export(rankToContigTab)
export(ratioAllComb)
export(ratioToPpm)
export(readCsvBatch)
export(readTabulatedBatch)
export(readVarColumns)
export(readXlsxBatch)
export(reduceTable)
export(regrBy1or2point)
export(regrMultBy1or2point)
export(renameColumns)
export(reorgByCluNo)
export(replNAbyLow)
export(replPlateCV)
export(replacePWseparator)
export(replicateStructure)
export(rmDupl2colMatr)
export(rmEnumeratorName)
export(rmOrphans)
export(rmSharedWords)
export(rnormW)
export(rowCVs)
export(rowGrpCV)
export(rowGrpMeans)
export(rowGrpNA)
export(rowGrpSds)
export(rowGrpSums)
export(rowMedSds)
export(rowNormalize)
export(rowSEMs)
export(rowSds)
export(sampNoDeMArrayLM)
export(scaleXY)
export(searchDataPairs)
export(searchLinesAtGivenSlope)
export(simpleFragFig)
export(singleLineAnova)
export(sortBy2CategorAnd1IntCol)
export(sortByNRepeated)
export(stableMode)
export(standardW)
export(stdErrMedBoot)
export(sumNAperGroup)
export(summarizeCols)
export(sysDate)
export(tTestAllVal)
export(tableToPlot)
export(test2factLimma)
export(top3mean)
export(transpGraySca)
export(treatTxtDuplicates)
export(triCoord)
export(trimRedundText)
export(trimmedMean)
export(unifyEnumerator)
export(uniqCountReport)
export(upperMaCoord)
export(vectorDistAngle)
export(volcanoFilter)
export(withinRefRange)
export(writeCsv)
importFrom(graphics,par)
